/*
 *
 * HWPtoPS - The Converter from HWP LQ-KSSM printing image to PostScript
 *
 * By Kwon Bomjun, SPARCS, KAIST
 *
 */

#include <stdio.h>
#include <stdlib.h>

#define LINEFEED 10
#define C_RETURN 13
#define FORMFEED 12

FILE *fp1, *fp2;
int x, y, l = 24, mx = 0, my = 0;
unsigned char buf[600][4000];

int bit[8] = {128, 64, 32, 16, 8, 4, 2, 1};

void Error(s)
char *s;
	{
	fprintf(stderr, "%s\n", s);
	exit(1);
	}

void MakePS()
	{
	int c = 0, x, y;

	x = mx + 1;
	y = my + 1;
	fprintf(fp2, "%%!PS-Adobe-2.0 EPSF-2.0\n");
	fprintf(fp2, "/bufstr %d string def\n", mx / 8 + 1);
	fprintf(fp2, "gsave\n");
	fprintf(fp2, "0 %f translate\n", (29.7 / 2.54 - y / 300.0) * 72);
	fprintf(fp2, "%f %f scale\n", x / 300.0 * 72, y / 300.0 * 72);
	fprintf(fp2, "%d %d 1\n", x, y);
	fprintf(fp2, "[%d 0 0 %d 0 %d]\n", x, -y, y);
	fprintf(fp2, "{currentfile bufstr readhexstring pop} bind\n");
	fprintf(fp2, "image\n");
	for (y = 0; y <= my; y++)
		{
		for (x = 0; x <= mx / 8; x++)
			{
				fprintf(fp2, "%02X", 255 ^ buf[x][y]);
			}
		fprintf(fp2, "\n");
		}
	fprintf(fp2, "grestore\n");
	fprintf(fp2, "showpage\n");
	fclose(fp1);
	fclose(fp2);
	}

void ControlCode()
	{
	unsigned char k;
	int i, n;
	static t = 0;

	k = getc(fp1);
	switch(k)
		{
		case '@':
			l = 24;
			return;
		case '3':
			k = getc(fp1);
			l = k;
			return;
		case 'U':
			if ((k = getc(fp1)) == '\1')
				fprintf(stderr, "UNI Direction\n");
			else if (k == '\0')
				fprintf(stderr, "BI Direction\n");
			else
				Error("Unknown Direction");
			return;
		case '*':
			if (getc(fp1) == '\'')
				{
				n = getc(fp1);
				n += getc(fp1) * 256;
				}
			else
				Error("Not 24pin 180dpi Code");
			fprintf(stderr, "line %d: %d points\n", ++t, n);
			while(n > 0)
				{
				k = getc(fp1);
				for (i = 7; i >= 0; i--)
					{
					if (k % 2 == 1)
						{
						buf[x / 8][y + i] |= bit[x % 8];
						}
					k /= 2;
					}
				k = getc(fp1);
				for (i = 15; i >= 8; i--)
					{
					if (k % 2 == 1)
						{
						buf[x / 8][y + i] |= bit[x % 8];
						}
					k /= 2;
					}
				k = getc(fp1);
				for (i = 23; i >= 16; i--)
					{
					if (k % 2 == 1)
						{
						buf[x / 8][y + i] |= bit[x % 8];
						}
					k /= 2;
					}
				x++;
				if (mx < x)
					mx = x;
				if (my < y + 23)
					my = y + 23;
				n--;
				}
			break;
		default:
			Error("Unknown ESC sequence");
		}
	}
			
void main(argc, argv)
int argc;
char *argv[];
	{
	char k;

	for (x = 0; x < 200; x++)
		for (y = 0; y < 2300; y++)
			buf[x][y] = 0;

	x = 0; y = 0;
	if (argc != 2 && argc != 3)
		{
		fprintf(stderr, "Usage: %s HWP_image_file [PostScript_file]\n", argv[0]);
		exit(1);
		}
	if ((fp1 = fopen(argv[1], "r")) == NULL)
		{
		fprintf(stderr, "%s: %s: No such file\n", argv[0], argv[1]);
		exit(1);
		}
	if (argc == 3)
		{
		if ((fp2 = fopen(argv[2], "w")) == NULL)
			{
			fprintf(stderr, "%s: %s: Cannot write on\n", argv[0], argv[2]);
			exit(1);
			}
		}
	else
		{
		fp2 = stdout;
		}
	k = getc(fp1);
	while(!feof(fp1))
		{
		switch(k)
			{
			case LINEFEED: y += l; break;
			case FORMFEED: goto EndLoop;
			case C_RETURN: x = 0; break;
			case '\033':   ControlCode(); break;
			default:       Error("Not ESC sequence"); break;
			}
		k = getc(fp1);
		}

	EndLoop:
	MakePS();
	}
